@@ -172,7 +172,7 @@ void drawWithoutReplacementFisherYates(std::vector<size_t>& result, std::mt19937
 
   // Skip indices
   for (size_t i = 0; i < skip.size(); ++i) {
-    result.erase(result.begin() + skip[skip.size() - 1 - i]);
+    result.erase(result.begin() + skip[i]);
   }
 
   // Draw without replacement using Fisher Yates algorithm
