@@ -211,17 +211,6 @@ void drawWithoutReplacementFisherYates(std::vector<size_t>& result, std::mt19937
 void drawWithoutReplacementFisherYates(std::vector<size_t>& result, std::mt19937_64& random_number_generator,
     size_t max, const std::vector<size_t>& skip, size_t num_samples);
 
-/**
- * Draw random numers without replacement and with weighted probabilites from vector of indices.
- * @param result Vector to add results to. Will not be cleaned before filling.
- * @param random_number_generator Random number generator
- * @param indices Vector with numbers to draw
- * @param num_samples Number of samples to draw
- * @param weights A weight for each element of indices
- */
-void drawWithoutReplacementWeighted(std::vector<size_t>& result, std::mt19937_64& random_number_generator,
-    const std::vector<size_t>& indices, size_t num_samples, const std::vector<double>& weights);
-
 /**
  * Draw random numers without replacement and with weighted probabilites from 0..n-1.
  * @param result Vector to add results to. Will not be cleaned before filling.
@@ -547,37 +536,6 @@ inline bool checkInterrupt() {
 }
 #endif
 
-// Provide make_unique (not available in C++11)
-namespace detail {
-
-template<class T> struct _Unique_if {
-  typedef std::unique_ptr<T> _Single_object;
-};
-
-template<class T> struct _Unique_if<T[]> {
-  typedef std::unique_ptr<T[]> _Unknown_bound;
-};
-
-template<class T, size_t N> struct _Unique_if<T[N]> {
-  typedef void _Known_bound;
-};
-
-} // namespace detail
-
-template<class T, class ... Args>
-typename detail::_Unique_if<T>::_Single_object make_unique(Args&&... args) {
-  return std::unique_ptr<T>(new T(std::forward<Args>(args)...));
-}
-
-template<class T>
-typename detail::_Unique_if<T>::_Unknown_bound make_unique(size_t n) {
-  typedef typename std::remove_extent<T>::type U;
-  return std::unique_ptr<T>(new U[n]());
-}
-
-template<class T, class ... Args>
-typename detail::_Unique_if<T>::_Known_bound make_unique(Args&&...) = delete;
-
 }
 // namespace ranger
 
